@@ -17,7 +17,7 @@ require HTML::DOM::Node;
 require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Node';
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 
 
 {
@@ -348,7 +348,7 @@ sub setAttribute {
 		my $eavesdropper = &$listener_maker(
 			$self, my $name = lc $1, $_[1]
 		);
-		defined $eavesdropper and $self-> _add_attr_event(
+		defined $eavesdropper and $self-> attr_event_listener(
 			$name, $eavesdropper
 		);
 	}
@@ -359,6 +359,7 @@ sub setAttribute {
 # This is just like attr, except that it triggers events.
 sub _attr {
 	my($self,$name) = (shift,shift);
+# ~~~ Can we change getAttribute to attr, to make it faster, or will attr reject a reference? (Do we have to stringify it?)
 	my $old = $self->getAttribute($name) if defined wantarray;
 	@_ and $self->setAttribute($name, shift);
 	$old;
@@ -461,7 +462,7 @@ sub setAttributeNode {
 		my $eavesdropper = &$listener_maker(
 			$_[0], $name = lc $1, $_[1]->nodeValue
 		);
-		defined $eavesdropper and $_[0]-> _add_attr_event(
+		defined $eavesdropper and $_[0]-> attr_event_listener(
 			$name, $eavesdropper
 		);
 	}
@@ -643,12 +644,6 @@ sub trigger_event {
 	);
 }
 
-sub _add_attr_event {
-	my ($self,$name,$listener) = @_;
-	$self->${\"on\L$name"}($listener);
-	return;
-}
-
 
 =head1 SEE ALSO
 
@@ -667,21 +662,21 @@ L<HTML::DOM/CLASSES AND DOM INTERFACES>
 # ------- HTMLHtmlElement interface ---------- #
 
 package HTML::DOM::Element::HTML;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub version { shift->_attr('version' => @_) }
 
 # ------- HTMLHeadElement interface ---------- #
 
 package HTML::DOM::Element::Head;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub profile { shift->_attr('profile' => @_) }
 
 # ------- HTMLLinkElement interface ---------- #
 
 package HTML::DOM::Element::Link;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub disabled {
 	if(@_ > 1) {
@@ -743,7 +738,7 @@ sub removeAttributeNode {
 # ------- HTMLTitleElement interface ---------- #
 
 package HTML::DOM::Element::Title;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 # This is what I call FWP (no lexical vars):
 sub text {
@@ -758,7 +753,7 @@ sub text {
 # ------- HTMLMetaElement interface ---------- #
 
 package HTML::DOM::Element::Meta;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub content   { shift->_attr('content'    => @_) }
 sub httpEquiv { shift->_attr('http-equiv' => @_) }
@@ -768,7 +763,7 @@ sub scheme    { shift->_attr('scheme'     => @_) }
 # ------- HTMLBaseElement interface ---------- #
 
 package HTML::DOM::Element::Base;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *href =\& HTML::DOM::Element::Link::href;
 *target =\& HTML::DOM::Element::Link::target;
@@ -776,7 +771,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLIsIndexElement interface ---------- #
 
 package HTML::DOM::Element::IsIndex;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub form     { (shift->look_up(_tag => 'form'))[0] || () }
 # ~~~ Should this be the same as Select::form? I.e., should isindex ele-
@@ -786,7 +781,7 @@ sub prompt   { shift->_attr('prompt'  => @_) }
 # ------- HTMLStyleElement interface ---------- #
 
 package HTML::DOM::Element::Style;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *disabled = \&HTML::DOM::Element::Link::disabled;
 *media =\& HTML::DOM::Element::Link::media;
@@ -806,7 +801,7 @@ sub sheet {
 # ------- HTMLBodyElement interface ---------- #
 
 package HTML::DOM::Element::Body;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub aLink      { shift->_attr( aLink      => @_) }
 sub background { shift->_attr( background => @_) }
@@ -814,6 +809,13 @@ sub bgColor    { shift->_attr('bgcolor'   => @_) }
 sub link       { shift->_attr('link'      => @_) }
 sub text       { shift->_attr('text'      => @_) }
 sub vLink      { shift->_attr('vlink'     => @_) }
+sub attr_event_listener {
+ my $self = shift;
+ my $target = $self->ownerDocument->event_parent;
+ $target
+  ? $target->attr_event_listener(@_)
+  : $self->SUPER::attr_event_listener(@_);
+}
 
 # ------- HTMLFormElement interface ---------- #
 
@@ -824,7 +826,7 @@ sub vLink      { shift->_attr('vlink'     => @_) }
 # ------- HTMLUListElement interface ---------- #
 
 package HTML::DOM::Element::UL;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub compact { shift->_attr( compact => @_) }
 sub type { lc shift->_attr( type => @_) }
@@ -832,7 +834,7 @@ sub type { lc shift->_attr( type => @_) }
 # ------- HTMLOListElement interface ---------- #
 
 package HTML::DOM::Element::OL;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub start { shift->_attr( start => @_) }
 *compact=\&HTML::DOM::Element::UL::compact;
@@ -841,28 +843,28 @@ sub start { shift->_attr( start => @_) }
 # ------- HTMLDListElement interface ---------- #
 
 package HTML::DOM::Element::DL;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLDirectoryElement interface ---------- #
 
 package HTML::DOM::Element::Dir;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLMenuElement interface ---------- #
 
 package HTML::DOM::Element::Menu;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLLIElement interface ---------- #
 
 package HTML::DOM::Element::LI;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *type =\& HTML::DOM::Element::Link::type;
 sub value { shift->_attr( value => @_) }
@@ -870,49 +872,49 @@ sub value { shift->_attr( value => @_) }
 # ------- HTMLDivElement interface ---------- #
 
 package HTML::DOM::Element::Div;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub align { lc shift->_attr( align => @_) }
 
 # ------- HTMLParagraphElement interface ---------- #
 
 package HTML::DOM::Element::P;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLHeadingElement interface ---------- #
 
 package HTML::DOM::Element::Heading;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLQuoteElement interface ---------- #
 
 package HTML::DOM::Element::Quote;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub cite { shift->_attr( cite => @_) }
 
 # ------- HTMLPreElement interface ---------- #
 
 package HTML::DOM::Element::Pre;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub width { shift->_attr( width => @_) }
 
 # ------- HTMLBRElement interface ---------- #
 
 package HTML::DOM::Element::Br;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub clear { lc shift->_attr( clear => @_) }
 
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::BaseFont;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub color { shift->_attr( color => @_) }
 sub face  { shift->_attr( face  => @_) }
@@ -921,7 +923,7 @@ sub size  { shift->_attr( size  => @_) }
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::Font;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *color =\& HTML::DOM::Element::BaseFont::color;
 *face =\& HTML::DOM::Element::BaseFont::face;
@@ -930,7 +932,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLHRElement interface ---------- #
 
 package HTML::DOM::Element::HR;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 sub noShade  { shift->_attr( noshade  => @_) }
@@ -940,7 +942,7 @@ sub noShade  { shift->_attr( noshade  => @_) }
 # ------- HTMLModElement interface ---------- #
 
 package HTML::DOM::Element::Mod;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *cite =\& HTML::DOM::Element::Quote::cite;
 sub dateTime  { shift->_attr( datetime  => @_) }
@@ -948,7 +950,7 @@ sub dateTime  { shift->_attr( datetime  => @_) }
 # ------- HTMLAnchorElement interface ---------- #
 
 package HTML::DOM::Element::A;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub accessKey  { shift->_attr(               accesskey  => @_) }
 *   charset    =\&HTML::DOM::Element::Link::charset           ;
@@ -978,10 +980,115 @@ sub trigger_event {
 	);
 }
 
+sub _get_abs_href {
+	my $elem = shift;
+	my $uri = new URI $elem->attr('href');
+	if(!$uri->scheme) {
+		my $base = $elem->ownerDocument->base;
+		return unless $base;
+		$uri = $uri->abs($base);
+		return unless $uri->scheme;
+	}
+	$uri
+}
+
+sub hash {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old;
+	if(defined wantarray) {
+		$old = $uri->fragment;
+		$old = "#$old" if defined $old;
+	}
+	if (@_){
+		shift() =~ /#?(.*)/s;
+		$uri->fragment($1);
+		$elem->_attr(href => $uri);
+	}
+	$old||''
+}
+
+sub host {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old = $uri->host_port if defined wantarray;
+	if (@_) {
+		$uri->port("");
+		$uri->host_port(shift);
+		$elem->attr(href => $uri);
+	}
+	$old
+}
+
+sub hostname {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old = $uri->host if defined wantarray;
+	if (@_) {
+		$uri->host(shift);
+		$elem->attr(href => $uri);
+	}
+	$old
+}
+
+sub pathname {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old = $uri->path if defined wantarray;
+	if (@_) {
+		$uri->path(shift);
+		$elem->attr(href => $uri);
+	}
+	$old
+}
+
+sub port {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old = $uri->port if defined wantarray;
+	if (@_) {
+		$uri->port(shift);
+		$elem->attr(href => $uri);
+	}
+	$old
+}
+
+sub protocol {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old = $uri->scheme . ':' if defined wantarray;
+	if (@_) {
+		shift() =~ /(.*):?/s;
+		$uri->scheme("$1");
+		$elem->attr(href => $uri);
+	}
+	$old
+
+}
+
+sub search {
+	my $elem = shift;
+	defined(my $uri = _get_abs_href $elem) or return '';
+	my $old;
+	if(defined wantarray) {
+		my $q = $uri->query;
+		$old = defined $q ? "?$q" : "";
+	}
+	if (@_){
+		shift() =~ /(\??)(.*)/s;
+		$uri->query(
+			$1||length$2 ? "$2" : undef
+		);
+		$elem->attr(href => $uri);
+	}
+	$old
+}
+
+
 # ------- HTMLImageElement interface ---------- #
 
 package HTML::DOM::Element::Img;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub lowSrc  { shift->attr(               lowsrc  => @_) }
 *   name  = \&HTML::DOM::Element::Meta::name            ;
@@ -1000,7 +1107,7 @@ sub vspace   { shift->_attr(              vspace   => @_) }
 # ------- HTMLObjectElement interface ---------- #
 
 package HTML::DOM::Element::Object;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *form=\&HTML::DOM::Element::Select::form;
 sub code  { shift->_attr(               code  => @_) }
@@ -1025,7 +1132,7 @@ sub contentDocument{}
 # ------- HTMLParamElement interface ---------- #
 
 package HTML::DOM::Element::Param;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *name=\&HTML::DOM::Element::Meta::name;
 *type=\&HTML::DOM::Element::Link::type;
@@ -1035,7 +1142,7 @@ sub valueType{lc shift->_attr(valuetype=>@_)}
 # ------- HTMLAppletElement interface ---------- #
 
 package HTML::DOM::Element::Applet;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 * align    = \ & HTML::DOM::Element::Div::align       ;
 * alt      = \ & HTML::DOM::Element::Img::alt         ;
@@ -1052,7 +1159,7 @@ sub object { shift -> _attr ( object => @_ ) }
 # ------- HTMLMapElement interface ---------- #
 
 package HTML::DOM::Element::Map;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub areas { # ~~~ I need to make this cache the resulting collection obj
 	my $self = shift;
@@ -1073,21 +1180,24 @@ sub areas { # ~~~ I need to make this cache the resulting collection obj
 # ------- HTMLAreaElement interface ---------- #
 
 package HTML::DOM::Element::Area;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
-* accessKey = \ & HTML::DOM::Element::A::accessKey     ;
 * alt       = \ & HTML::DOM::Element::Img::alt         ;
 sub coords { shift -> _attr ( coords => @_ ) }
 * href      = \ & HTML::DOM::Element::Link::href       ;
 sub noHref { shift -> attr ( nohref => @_ ) }
-* shape     = \ & HTML::DOM::Element::A::shape         ;
 * tabIndex  = \ & HTML::DOM::Element::Object::tabIndex ;
 * target    = \ & HTML::DOM::Element::Link::target     ;
+{
+ no strict 'refs';
+ *$_ = \&{"HTML::DOM::Element::A::$_"}
+  for qw(accessKey shape hash host hostname pathname port protocol search);
+}
 
 # ------- HTMLScriptElement interface ---------- #
 
 package HTML::DOM::Element::Script;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 * text    = \ &HTML::DOM::Element::Title::text   ;
 sub htmlFor { shift -> _attr ( for   => @_ )      }
@@ -1100,7 +1210,7 @@ sub defer   { shift -> _attr ( defer => @_ )      }
 # ------- HTMLFrameSetElement interface ---------- #
 
 package HTML::DOM::Element::FrameSet;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub rows { shift -> _attr ( rows   => @_ )      }
 sub cols   { shift -> _attr ( cols => @_ )      }
@@ -1108,7 +1218,7 @@ sub cols   { shift -> _attr ( cols => @_ )      }
 # ------- HTMLFrameElement interface ---------- #
 
 package HTML::DOM::Element::Frame;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 sub frameBorder { lc shift -> _attr ( frameBorder  => @_ )      }
 sub longDesc    { shift -> _attr ( longdesc     => @_ )      }
@@ -1128,7 +1238,7 @@ sub contentWindow {
 # ------- HTMLIFrameElement interface ---------- #
 
 package HTML::DOM::Element::IFrame;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Div::align;
 *frameBorder = \&HTML::DOM::Element::Frame::frameBorder;
